/* Some misc util functions for audio DSP work, written by Steve Harris,
   December 2000

   steve@plugin.org.uk
*/

#ifndef LADSPA_UTIL_H
#define LADSPA_UTIL_H

// Denormalise floats, only actually needed for PIII and very recent PowerPC
#define FLUSH_TO_ZERO(fv) (((*(unsigned int*)&(fv))&0x7f800000)==0)?0.0f:(fv)

// Limit a value to be l<=v<=u
#define LIMIT(v,l,u) ((v)<(l)?(l):((v)>(u)?(u):(v)))

// Truncate-to-zero modulo (ANSI C doesn't specify) will only work
// if v > -m and v < 2m
#define MOD(v,m) (v<0?v+m:(v>=m?v-m:v))

// Truncate-to-zero modulo (ANSI C doesn't specify) will only work
// if v > -m and v < m
#define NEG_MOD(v,m) ((v)<0?((v)+(m)):(v))

// Convert a value in dB's to a coefficent
#define DB_CO(g) ((g) > -90.0f ? pow(10.0f, (g) * 0.05f) : 0.0f)

// Linearly interpolate
#define LIN_INTERP(f,a,b) ((1.0f - f) * (a) + (f) * (b))

// Cubic interpolation function
static inline float cube_interp(const float fr, const float inm1, const float in, const float inp1, const float inp2)
{
	return in + 0.5f * fr * (inp1 - inm1 +
	 fr * (4.0f * inp1 + 2.0f * inm1 - 5.0f * in - inp2 +
	 fr * (3.0f * (in - inp1) - inm1 + inp2)));
}

// Truncate float to int
static inline int f_trunc(float f) {
        f -= 0.5f;
        f += (3<<22);
        return *((int*)&f) - 0x4b400000;
}

// Round float to int (faster than f_trunc)
static inline int f_round(float f) {
        f += (3<<22);
        return *((int*)&f) - 0x4b400000;
}

// 16.16 fixpoint
typedef union {
	int all;
	struct {
		unsigned short fr;
		short in;
	} part;
} fixp16;

// 32.32 fixpoint
typedef union {
	long long all;
	struct {
		unsigned int fr;
		int in;
	} part;
} fixp32;

#endif
